@@ -399,9 +399,24 @@ class Instruction:
     def trace_stack_position(
         self, stack_position: int
     ) -> typing.Iterator["Instruction"]:
-        yielded = {self}
-        for instr in self.previous_instructions:
-            yield from instr._trace_stack_position(stack_position, yielded, self)
+        for instr in self.get_priorities_previous():
+            yield from instr._trace_stack_position(stack_position, set(), self)
+
+    def get_priorities_previous(self) -> typing.List["Instruction"]:
+        if not self.previous_instructions: return []
+
+        real_previous = [instr for instr in self.previous_instructions if instr.next_instruction == self]
+        unreal_previous = [instr for instr in self.previous_instructions if instr.next_instruction != self]
+        highest = None
+
+        for e in real_previous:
+            if e.offset == self.offset - 1 or e.opcode == Opcodes.SETUP_FINALLY:
+                highest = e
+
+        if not highest:
+            return real_previous + unreal_previous
+
+        return [highest] + [e for e in real_previous if e != highest] + unreal_previous
 
     def trace_normalized_stack_position(self, stack_position: int) -> typing.Optional["Instruction"]:
         target = next(self.trace_stack_position(stack_position))
@@ -431,15 +446,27 @@ class Instruction:
             pushed, popped, additional_pos = 0, 0, None
         elif self.opcode == Opcodes.SETUP_FINALLY and previous_instr == self.arg_value:
             pushed, popped, additional_pos = 3, 0, None
+        elif self.opcode in (
+            Opcodes.JUMP_IF_TRUE_OR_POP,
+            Opcodes.JUMP_IF_FALSE_OR_POP,
+        ):
+            if self.arg_value == previous_instr:
+                pushed, popped, additional_pos = 0, 0, None
+            else:
+                pushed, popped, additional_pos = 0, 1, None
         else:
             pushed, popped, additional_pos = self.get_stack_affect()
 
+        # print(self, stack_position, (pushed, popped, additional_pos), self.arg_value, previous_instr)
+
         if pushed > stack_position:
             yielded.add(self)
+            # print("hit")
             yield self
+            # print("cont")
 
             if additional_pos:
-                for instr in self.previous_instructions:
+                for instr in self.get_priorities_previous():
                     yield from instr._trace_stack_position(additional_pos, yielded, self)
 
             return
@@ -449,7 +476,7 @@ class Instruction:
 
         yielded.add(self)
 
-        for instr in self.previous_instructions or tuple():
+        for instr in self.get_priorities_previous():
             try:
                 yield from instr._trace_stack_position(stack_position, yielded, self)
             except:
@@ -457,13 +484,19 @@ class Instruction:
                 raise
 
         if additional_pos:
-            for instr in self.previous_instructions:
+            for instr in self.get_priorities_previous():
                 yield from instr._trace_stack_position(additional_pos, yielded, self)
 
     def trace_stack_position_use(
         self, stack_position: int
     ) -> typing.Iterator["Instruction"]:
-        yield from self._trace_stack_position_use(stack_position + 1, set())
+        print(self, 0)
+
+        if not self.has_stop_flow():
+            yield from self.next_instruction._trace_stack_position_use(stack_position, set())
+
+        if self.has_jump():
+            yield from typing.cast(Instruction, self.arg_value)._trace_stack_position_use(stack_position, set())
 
     def _trace_stack_position_use(
         self, stack_position: int, yielded: typing.Set["Instruction"]
@@ -473,6 +506,8 @@ class Instruction:
         if self in yielded:
             return
 
+        yielded.add(self)
+
         pushed, popped, additional_pos = self.get_stack_affect()
 
         if popped > stack_position:
@@ -480,24 +515,25 @@ class Instruction:
             yield self
 
             if additional_pos:
-                for instr in self.previous_instructions:
-                    yield from instr._trace_stack_position(additional_pos, yielded)
+                yield from self.next_instruction._trace_stack_position_use(additional_pos, yielded)
 
             return
 
         stack_position -= popped
         stack_position += pushed
 
-        for instr in self.previous_instructions:
-            yield from instr._trace_stack_position(stack_position, yielded)
+        if not self.has_stop_flow():
+            yield from self.next_instruction._trace_stack_position_use(stack_position, yielded)
+
+        if self.has_jump():
+            yield from typing.cast(Instruction, self.arg_value)._trace_stack_position_use(stack_position, yielded)
 
         if additional_pos:
-            for instr in self.previous_instructions:
-                yield from instr._trace_stack_position(additional_pos, yielded)
+            yield from self.next_instruction._trace_stack_position_use(additional_pos, yielded)
 
     def get_stack_affect(self) -> typing.Tuple[int, int, int | None]:
         # pushed, popped, additional
-        if self.opcode in (Opcodes.NOP, Opcodes.POP_BLOCK, Opcodes.POP_EXCEPT, Opcodes.SETUP_FINALLY):
+        if self.opcode in (Opcodes.NOP, Opcodes.POP_BLOCK, Opcodes.POP_EXCEPT, Opcodes.SETUP_FINALLY, Opcodes.GEN_START, Opcodes.JUMP_ABSOLUTE):
             return 0, 0, None
 
         if self.opcode == Opcodes.DUP_TOP:
@@ -511,30 +547,49 @@ class Instruction:
             Opcodes.LOAD_GLOBAL,
             Opcodes.LOAD_FAST,
             Opcodes.LOAD_DEREF,
+            Opcodes.LOAD_CLOSURE,
         ):
             return 1, 0, None
 
         if self.opcode in (
-            Opcodes.CALL_FUNCTION,
-            Opcodes.CALL_METHOD,
             Opcodes.BUILD_TUPLE,
             Opcodes.BUILD_LIST,
             Opcodes.BUILD_SET,
             Opcodes.BUILD_SLICE,
+            Opcodes.BUILD_STRING,
         ):
             return 1, self.arg, None
 
         if self.opcode in (
+            Opcodes.ROT_TWO,
+        ):
+            return 2, 2, None
+
+        if self.opcode in (
+            Opcodes.FORMAT_VALUE,
+        ):
+            return 1, (2 if (self.arg & 0x04) == 0x04 else 1), None
+
+        if self.opcode in (
+            Opcodes.CALL_FUNCTION,
+            Opcodes.CALL_METHOD,
             Opcodes.CALL_FUNCTION_KW,
         ):
             return 1, self.arg + 1, None
 
+        if self.opcode in (
+            Opcodes.CALL_FUNCTION_EX,
+        ):
+            count = 2
+
+            if self.arg & 0x01:
+                count += 1
+
+            return 1, count, None
+
         if self.opcode == Opcodes.BUILD_MAP:
             return 1, self.arg * 2, None
 
-        if self.opcode == Opcodes.RETURN_VALUE:
-            raise ValueError(self)
-
         if self.opcode in (
             Opcodes.COMPARE_OP,
             Opcodes.IS_OP,
@@ -548,13 +603,27 @@ class Instruction:
             Opcodes.DICT_MERGE,
             Opcodes.BINARY_FLOOR_DIVIDE,
             Opcodes.BINARY_ADD,
+            Opcodes.BINARY_SUBTRACT,
             Opcodes.BINARY_MULTIPLY,
+            Opcodes.BINARY_TRUE_DIVIDE,
+            Opcodes.BINARY_FLOOR_DIVIDE,
+            Opcodes.BINARY_MODULO,
+            Opcodes.BINARY_XOR,
+            Opcodes.BINARY_AND,
+            Opcodes.BINARY_OR,
+            Opcodes.BINARY_POWER,
+            Opcodes.INPLACE_SUBTRACT,
+            Opcodes.IMPORT_NAME,
         ):
             return 1, 2, None
 
         if self.opcode in (
             Opcodes.LOAD_ATTR,
             Opcodes.LOAD_METHOD,
+            Opcodes.GET_AWAITABLE,
+            Opcodes.LIST_TO_TUPLE,
+            Opcodes.IMPORT_FROM,
+            Opcodes.UNARY_NEGATIVE,
         ):
             return 1, 1, None
 
@@ -566,7 +635,12 @@ class Instruction:
         if self.opcode in (
             Opcodes.POP_TOP,
             Opcodes.POP_JUMP_IF_TRUE,
+            Opcodes.POP_JUMP_IF_FALSE,
             Opcodes.STORE_FAST,
+            Opcodes.STORE_DEREF,
+            Opcodes.YIELD_VALUE,
+            Opcodes.YIELD_FROM,
+            Opcodes.RETURN_VALUE,
         ):
             return 0, 1, None
 
@@ -576,6 +650,9 @@ class Instruction:
         if self.opcode == Opcodes.JUMP_IF_NOT_EXC_MATCH:
             return 0, 2, None
 
+        if self.opcode == Opcodes.MAKE_FUNCTION:
+            return 1, 2 + self.arg.bit_count(), None
+
         raise RuntimeError(self)
 
     def insert_after(self, *instructions: "Instruction" | typing.List["Instruction"]):
@@ -589,7 +666,10 @@ class Instruction:
 
         instructions[0].next_instruction = self.next_instruction
         self.next_instruction = instructions[0]
-        instructions[0].insert_after(instructions[1:])
+
+        if len(instructions) > 1:
+            instructions[0].insert_after(instructions[1:])
+
         return self
 
 
@@ -598,10 +678,13 @@ class MutableFunction:
     def create(cls, target):
         if isinstance(target, staticmethod):
             return cls(target.__func__)
+        elif isinstance(target, classmethod):
+            return cls(target.__func__)
         return cls(target)
 
     def __init__(self, target: types.FunctionType | types.MethodType):
         self.target = target
+
         self.code_object: types.CodeType = target.__code__
 
         self.argument_count = self.code_object.co_argcount
@@ -720,6 +803,10 @@ class MutableFunction:
         self.prepare_previous_instructions()
 
     def prepare_previous_instructions(self):
+        for instruction in self.instructions:
+            if instruction.previous_instructions:
+                instruction.previous_instructions.clear()
+
         for instruction in self.instructions:
             if instruction.has_stop_flow() or instruction.has_unconditional_jump():
                 continue
@@ -764,11 +851,6 @@ class MutableFunction:
 
         # For a last time, walk over the instructions and tie them together
         for i, instruction in enumerate(instructions[:]):
-            # print(i, instruction.repr_safe())
-            #
-            # if isinstance(instruction.arg_value, Instruction):
-            #     print(i, "+", instruction.arg_value.repr_safe())
-
             if (
                 not instruction.has_stop_flow()
                 and not instruction.has_unconditional_jump()
@@ -776,19 +858,7 @@ class MutableFunction:
                 try:
                     instruction.next_instruction = instructions[i + 1]
                 except IndexError:
-                    pass
-                # except IndexError:
-                #     if not instruction.has_stop_flow() and instruction.next_instruction in instructions and i == len(instructions) - 1:
-                #         print("error", instruction, instruction.next_instruction)
-                #         instr = Instruction(self, i+1, Opcodes.JUMP_ABSOLUTE, arg=instruction.next_instruction.offset, _decode_next=False)
-                #         instructions.append(instr)
-                #         instr._next_instruction = instruction.next_instruction
-                #         instruction.next_instruction.add_previous_instruction(instr)
-                #         instruction._next_instruction = instr
-                #         instr.add_previous_instruction(instruction)
-                #         print(instruction.next_instruction.previous_instructions)
-                #     else:
-                #         raise
+                    instruction.next_instruction = None
                 except:
                     print(instruction)
                     print(instruction.next_instruction)
@@ -863,6 +933,19 @@ class MutableFunction:
             if instruction.has_stop_flow():
                 continue
 
+            # FOR_ITER can only jump FORWARD, so insert a new jump absolute to jump there
+            # TODO: insert the JUMP somewhere else than here!
+            if instruction.opcode == Opcodes.FOR_ITER:
+                jump = Instruction(
+                    self,
+                    -1,
+                    "JUMP_ABSOLUTE",
+                    instruction.next_instruction,
+                )
+                jump.change_arg_value(instruction.arg_value)
+                instruction.change_arg_value(jump)
+                instructions.append(jump)
+
             if (
                 instruction.next_instruction is not None
                 and instruction.offset + 1 != instruction.next_instruction.offset
