@@ -20,6 +20,8 @@ from bytecodemanipulation.util import _is_parent_of
 
 BUILTIN_CACHE = builtins.__dict__.copy()
 
+DISABLE_OPTIMISATION_APPLY = os.environ.setdefault("DISABLE_OPTIMISATION_APPLY", "")
+
 
 class ValueIsNotArrivalException(Exception):
     pass
@@ -34,7 +36,13 @@ class GlobalIsNeverAccessedException(Exception):
 
 
 class _OptimisationContainer:
-    _CUSTOM_TARGETS = {}
+    _CUSTOM_TARGETS: typing.Dict[typing.Hashable, "_OptimisationContainer"] = {}
+    _CONTAINERS: typing.List["_OptimisationContainer"] = []
+
+    @classmethod
+    def apply_all(cls):
+        for target in cls._CONTAINERS:
+            target.run_optimisers(False)
 
     @classmethod
     def get_for_target(cls, target):
@@ -47,6 +55,8 @@ class _OptimisationContainer:
 
         container = cls(target)
 
+        cls._CONTAINERS.append(container)
+
         try:
             target._OPTIMISER_CONTAINER = container
         except (AttributeError,  TypeError):
@@ -120,6 +130,18 @@ class _OptimisationContainer:
 
         self.is_optimized = False
 
+        # Add the children to self
+        if isinstance(self.target, type):
+            for key, e in self.target.__dict__.items():
+                if isinstance(e, (type(guarantee_builtin_names_are_protected), staticmethod, classmethod, type)):
+                    if isinstance(e, (staticmethod, classmethod)):
+                        func = e.__func__
+                    else:
+                        func = e
+
+                    if func.__qualname__.startswith(self.target.__qualname__+".") and func.__module__ == self.target.__module__:
+                        self.children.append(self.get_for_target(e))
+
     def is_attribute_static(self, name: str):
         return self.is_static or name in self.static_attributes
 
@@ -162,27 +184,33 @@ class _OptimisationContainer:
         self.optimisations.append(optimiser)
         return self
 
-    def run_optimisers(self):
+    def run_optimisers(self, warn=True):
         if self.is_optimized:
-            print("opt skipped", self.target)
+            if warn and not DISABLE_OPTIMISATION_APPLY:
+                print("opt skipped", self.target)
+
             return
 
         self.is_optimized = True
 
+        if DISABLE_OPTIMISATION_APPLY:
+            return
+
         print("opt", self.target)
         from bytecodemanipulation.optimiser_util import apply_specializations
 
-        if isinstance(self.target, type):
+        if self.children:
             self._walk_children_and_copy_attributes()
 
             for child in self.children:
                 child.run_optimisers()
 
-            return
-
         # resolve the lazy types
         self._resolve_lazy_references()
 
+        if not hasattr(self.target, "__code__") and not isinstance(self.target, (classmethod, staticmethod)):
+            return
+
         # Create mutable wrapper around the target
         mutable = MutableFunction.create(self.target)
         mutable.prepare_previous_instructions()
@@ -190,46 +218,56 @@ class _OptimisationContainer:
         # Walk over the code and resolve cached globals
         self._inline_load_globals(mutable)
 
-        dirty = True
-
         # Walk over the entries as long as the optimisers are doing their stuff
-        while dirty:
+        while True:
+            mutable.assemble_instructions_from_tree(
+                mutable.instructions[0].optimise_tree()
+            )
+            mutable.prepare_previous_instructions()
+
             dirty = False
+            for optimiser in self.optimisations:
+                dirty = optimiser.apply(self, mutable) or dirty
 
-            dirty = inline_const_value_pop_pairs(mutable) or dirty
+            if dirty:
+                continue
 
-            dirty = remove_local_var_assign_without_use(mutable) or dirty
+            if inline_const_value_pop_pairs(mutable):
+                continue
+
+            if remove_local_var_assign_without_use(mutable):
+                continue
 
             # Inlines access to static items
-            dirty = inline_static_attribute_access(mutable) or dirty
+            if inline_static_attribute_access(mutable):
+                continue
 
             # Inline invokes to builtins and other known is_constant_op-s with static args
-            dirty = inline_constant_method_invokes(mutable) or dirty
+            if inline_constant_method_invokes(mutable):
+                continue
 
             # Resolve known constant types
-            dirty = self._resolve_constant_local_types(mutable) or dirty
+            if self._resolve_constant_local_types(mutable):
+                continue
             # self._resolve_constant_local_attr_types(mutable)
             # todo: use return type of known functions
 
-            dirty = inline_constant_binary_ops(mutable) or dirty
+            if inline_constant_binary_ops(mutable):
+                continue
 
             # apply optimisation specialisations
-            dirty = apply_specializations(mutable) or dirty
+            if apply_specializations(mutable):
+                continue
 
             # Inline invokes to builtins and other known is_constant_op-s with static args
-            dirty = inline_constant_method_invokes(mutable) or dirty
+            if inline_constant_method_invokes(mutable):
+                continue
 
             # Remove conditional jumps no longer required
-            dirty = remove_branch_on_constant(mutable) or dirty
+            if remove_branch_on_constant(mutable):
+                continue
 
-            if dirty:
-                mutable.assemble_instructions_from_tree(
-                    mutable.instructions[0].optimise_tree()
-                )
-                mutable.prepare_previous_instructions()
-
-            for optimiser in self.optimisations:
-                dirty = optimiser.apply(self, mutable) or dirty
+            break
 
         mutable.assemble_instructions_from_tree(mutable.instructions[0].optimise_tree())
 
@@ -238,23 +276,32 @@ class _OptimisationContainer:
         # print(mutable)
 
     def _resolve_lazy_references(self):
-        for key, lazy in self.lazy_global_name_cache.items():
-            if key not in self.dereference_global_name_cache:
-                self.dereference_global_name_cache[key] = lazy()
-        for key, lazy in self.lazy_local_var_type.items():
-            if key not in self.dereference_local_var_type:
-                self.dereference_local_var_type[key] = lazy()
-        for key, lazy in self.lazy_local_var_attr_type.items():
-            if key not in self.dereference_local_var_attr_type:
-                self.dereference_local_var_attr_type[key] = lazy()
-        if self.dereference_return_type is None and self.lazy_return_type is not None:
-            self.dereference_return_type = self.lazy_return_type()
-        for key, lazy in self.lazy_return_attr_type.items():
-            if key not in self.dereference_return_attr_type:
-                self.dereference_return_attr_type[key] = lazy()
-        for key, lazy in self.lazy_attribute_type.items():
-            if key not in self.dereference_attribute_type:
-                self.dereference_attribute_type[key] = lazy()
+        try:
+            for key, lazy in self.lazy_global_name_cache.items():
+                if key not in self.dereference_global_name_cache:
+                    self.dereference_global_name_cache[key] = lazy()
+
+            for key, lazy in self.lazy_local_var_type.items():
+                if key not in self.dereference_local_var_type:
+                    self.dereference_local_var_type[key] = lazy()
+
+            for key, lazy in self.lazy_local_var_attr_type.items():
+                if key not in self.dereference_local_var_attr_type:
+                    self.dereference_local_var_attr_type[key] = lazy()
+
+            if self.dereference_return_type is None and self.lazy_return_type is not None:
+                self.dereference_return_type = self.lazy_return_type()
+
+            for key, lazy in self.lazy_return_attr_type.items():
+                if key not in self.dereference_return_attr_type:
+                    self.dereference_return_attr_type[key] = lazy()
+
+            for key, lazy in self.lazy_attribute_type.items():
+                if key not in self.dereference_attribute_type:
+                    self.dereference_attribute_type[key] = lazy()
+        except:
+            print(self.target)
+            raise
 
     def _inline_load_globals(self, mutable: MutableFunction) -> bool:
         dirty = False
@@ -353,6 +400,12 @@ def cache_global_name(
 
     def annotate(target):
         container = _OptimisationContainer.get_for_target(target)
+
+        nonlocal accessor
+
+        if accessor is None:
+            accessor = lambda: target.__globals__.get(name)
+
         container.lazy_global_name_cache[name] = accessor
 
         if ignore_unsafe_writes:
@@ -635,6 +688,8 @@ def apply_now():
     """
 
     def annotate(target):
+        # dis.dis(target)
+
         container = _OptimisationContainer.get_for_target(target)
         container.run_optimisers()
 
