@@ -60,14 +60,27 @@ def inline_const_value_pop_pairs(mutable: MutableFunction) -> bool:
 
     for instruction in mutable.instructions:
         if instruction.opcode == Opcodes.POP_TOP:
-            source = next(instruction.trace_stack_position(0))
+            try:
+                # print("search")
+                source = next(instruction.trace_stack_position(0))
+            except StopIteration:
+                # source = next(instruction.trace_stack_position(0))
+
+                print("---")
+
+                for instr in mutable.instructions:
+                    print(instr)
+
+                print("---")
+                print(instruction)
+                print(instruction.get_priorities_previous())
+                raise
 
             # Inline LOAD_XX - POP pairs
             if source.opcode in SIDE_EFFECT_FREE_LOADS:
                 instruction.change_opcode(Opcodes.NOP)
                 source.change_opcode(Opcodes.NOP)
-                dirty = True
-                continue
+                return True
 
             # Inline CALL_XX (constant expr) - POP pairs
             if source.opcode == Opcodes.CALL_FUNCTION:
@@ -332,7 +345,7 @@ def remove_branch_on_constant(mutable: MutableFunction) -> bool:
     dirty = False
 
     for instruction in mutable.instructions:
-        if instruction.has_jump() and not instruction.has_unconditional_jump():
+        if instruction.has_jump() and not instruction.has_unconditional_jump() and instruction.opcode not in (Opcodes.SETUP_FINALLY, Opcodes.SETUP_WITH):
             if instruction.previous_instructions is None:
                 if instruction.offset == 0 and instruction.opcode == Opcodes.SETUP_FINALLY:
                     continue
@@ -401,6 +414,14 @@ def inline_static_attribute_access(mutable: MutableFunction) -> bool:
                     source_instr.change_opcode(Opcodes.NOP)
                     instruction.change_opcode(Opcodes.LOAD_CONST)
                     instruction.change_arg_value(getattr(source, attr_name))
+
+                    print(instruction)
+
+                    use = next(instruction.trace_stack_position_use(0))
+
+                    if use.opcode == Opcodes.CALL_METHOD:
+                        use.change_opcode(Opcodes.CALL_FUNCTION)
+
                     dirty = True
 
     return dirty
